// WebNumDial.cpp: implementation of the CWebNumDial class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <afxtempl.h>
#include <afxmt.h>
#include <afxwin.h>
#include "crobot.h"
#include "crobotinternet.h"
#include "HotOffThePress.h"
#include "DeviceIDDlg.h"
#include "SouceDlg.h"
#include "TimeTickerDlg.h"
#include "NewzDlg.h"
#include "FinancialDlg.h"
#include "AlertsDlg.h"
#include "UpdatesDlg.h"
#include "NoDeviceDlg.h"
#include "DownloadDlg.h"
#include "FindNumDlg.h"
#include "CalibNumDlg.h"
#include "AlignDlg.h"
#include "Ports.h"
#include "Device.h"
#include "DevicesDlg.h"
#include "WebNumDial.h"

#define VERSIONNO 1

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IMPLEMENT_SERIAL( CWebNumDial, CDevice, VERSIONNO );

// CWebNumDial yahoo ticker display

CWebNumDial::CWebNumDial()
{
	// Propeties
	m_DeviceName.LoadString( IDS_WEBNUMDIAL );
	m_DeviceType = DT_WEBNUMDIAL;

	m_strDeviceID.LoadString( IDS_DEFID );
	m_strURL.LoadString( IDS_DEFURL );
	m_iUpdate = 10;
	m_ResultText.LoadString( IDS_NORESULTTEXT );

	// Add date to ticker text for niceness.
	// Updates
	CTime t( 1999, 1, 1, 1, 1, 1 );
	m_LastWebUpdate = t;

	m_bOnFail = false;
	m_iHiDeg = 90;
	m_dHiVal = 100;
	m_iLoDeg = -90;
	m_dLoVal = 0;
	m_iOnFail = 180;
	m_bCalcPercent = false;
	m_bBefore = false;
	m_sTriggerText = "Temperature";

	m_bIsClock = false;
	m_bEuroFormat = false;
	m_iNumCount = 1;

	// iResult is bad until proven good
	m_iResult = 1;
}

CWebNumDial::CWebNumDial( CWebNumDial& yt)
{
	// Device stuff
	m_DeviceName = yt.m_DeviceName;
	m_DeviceType = yt.m_DeviceType;
	m_ResultText = yt.m_ResultText;
	m_strDeviceID = yt.m_strDeviceID;
	m_strURL = yt.m_strURL;
	m_iUpdate = yt.m_iUpdate;
	m_bSleep = yt.m_bSleep;
	m_Wake = yt.m_Wake;
	m_Sleep = yt.m_Sleep;
	strcpy( m_chid , yt.m_chid);

	m_bOnFail = yt.m_bOnFail;
	m_iHiDeg = yt.m_iHiDeg;
	m_dHiVal = yt.m_dHiVal;
	m_iLoDeg = yt.m_iLoDeg;
	m_dLoVal = yt.m_dLoVal;
	m_iOnFail = yt.m_iOnFail;
	m_bCalcPercent = yt.m_bCalcPercent;
	m_bBefore = yt.m_bBefore;
	m_sTriggerText = yt.m_sTriggerText;
	m_bIsClock = yt.m_bIsClock;
	m_bEuroFormat = yt.m_bEuroFormat;
	m_iNumCount = yt.m_iNumCount;

	// Propeties
	m_LastWebUpdate = yt.m_LastWebUpdate;
	m_iResult = yt.m_iResult;
}

CWebNumDial::~CWebNumDial()
{

}


void CWebNumDial::AddPages()
{
	// Add property pages to PropSheet
	CDevicesDlg* pDlg = ((CHotOffThePressApp*)AfxGetApp())->m_pDevicesDlg;
	pDlg->AddPage( &m_DeviceIDDlg );
	pDlg->AddPage( &m_SourceDlg );
//	pDlg->AddPage( &m_DownloadDlg );
	pDlg->AddPage( &m_FindNumDlg );
	pDlg->AddPage( &m_CalibNumDlg );
	pDlg->AddPage( &m_AlignDlg );
	pDlg->AddPage( &m_UpdatesDlg );
}

void CWebNumDial::RemovePages()
{
	// Add property pages to PropSheet
	CDevicesDlg* pDlg = ((CHotOffThePressApp*)AfxGetApp())->m_pDevicesDlg;
	pDlg->RemovePage( &m_DeviceIDDlg );
	pDlg->RemovePage( &m_SourceDlg );
//	pDlg->RemovePage( &m_DownloadDlg );
	pDlg->RemovePage( &m_FindNumDlg );
	pDlg->RemovePage( &m_CalibNumDlg );
	pDlg->RemovePage( &m_AlignDlg );
	pDlg->RemovePage( &m_UpdatesDlg );
}

void CWebNumDial::UpdateNow()
{
	// Polls web page IN THIS THREAD
	int RetVal;
	CString sDoc, sErr;

	// Poll web page
	((CHotOffThePressApp*)AfxGetApp())->m_Internet.httpGet(m_strURL, sDoc, RetVal, sErr);

	// Translate web page
	long lRes;
	ParseWebPage(	m_ResultText, lRes, sDoc, RetVal, sErr );
	m_iResult = lRes;
}


void CWebNumDial::ParseWebPage( CString& sResult, long &iResult, CString& sDoc, int RetVal, CString& sErr )
{
	// Remove HTML if desired
	if (!m_bCalcPercent) RemoveHTML(sDoc);

	// Search for trigger text
	long Pos = sDoc.Find( m_sTriggerText);
	if (Pos<0) 
	{
		sResult.LoadString( IDS_CANTFINDTRIGGER );
		if (m_bOnFail)
		{
			iResult = m_iOnFail;
		}
		return;
	}


	// Find number
	bool bIsNeg = false;
	CString str;
	CString numstr;
	if (m_bBefore)
	{
		str = sDoc.Left(Pos);
		str.MakeReverse();

		for (int nc=0; nc<m_iNumCount; nc++)
		{
			ASSERT (!this->m_bEuroFormat);
			// Scan back to find first digit or point
			str = str.Mid( str.FindOneOf( "1234567890." ) );

			// Extract number
			numstr = str.SpanIncluding("1234567890.");

			// Check not omre than one point
			int iPointCount = 0;
			for (long i=0; i<numstr.GetLength() && iPointCount<2; i++)
			{
				if (numstr.GetAt(i)=='.') iPointCount++;
			}
			if (iPointCount>1)
			{
				numstr = numstr.Left(i-1);
			}

			// Pare original string down 
			str = str.Mid( numstr.GetLength() );

			// if just extracted a dot, try again
			if (numstr==".") nc--;
		}
		numstr.MakeReverse();

		// crap out if not found
		if (numstr.GetLength()==0)
		{
			sResult.LoadString( IDS_CANTFINDNUMBEFORE );
			if (m_bOnFail)
			{
				iResult = m_iOnFail;
			}
			return;
		}

		// If starts with ".", ignore point
		if (numstr.GetAt(0)=='.') numstr = numstr.Mid(1);

		// Inspect pared string for minus sign
		if (str.GetLength()>0 && str.GetAt( 0 )=='-')  bIsNeg = true;
	}
	else
	{
		str = sDoc.Mid( Pos + m_sTriggerText.GetLength() );
		for (int nc=0; nc<m_iNumCount; nc++)
		{
			// find number
			int Loc = str.FindOneOf("1234567890");
			if (str.GetAt( Loc>1?Loc-1:0) == '-') bIsNeg = true;
			str = str.Mid( str.FindOneOf("1234567890") );

			// Extract number
			numstr = str.SpanIncluding("1234567890.");

			// Check not omre than one point
			int iPointCount = 0;
			for (long i=0; i<numstr.GetLength() && iPointCount<2; i++)
			{
				if (numstr.GetAt(i)=='.') iPointCount++;
			}
			if (iPointCount>1)
			{
				numstr = numstr.Left(i-1);
			}

			// crap out if not found
			if (numstr.GetLength()==0)
			{
				sResult.LoadString( IDS_CANTFINDNUMAFTER );
				if (m_bOnFail)
				{
					iResult = m_iOnFail;
				}
				return;
			}

			// Pare original string down 
			str = str.Mid( numstr.GetLength() );

			// if just extracted a dot, try again
			if (numstr==".") nc--;
		}

	}

	double dVal = 0;
	int iRetVal = sscanf( numstr, "%lf", &dVal );

	// If can't find number...
	if (iRetVal!=1)
	{
		if (m_bBefore)
		{
			sResult.LoadString( IDS_CANTFINDNUMBEFORE );
		}
		else
		{
			sResult.LoadString( IDS_CANTFINDNUMAFTER );
		}
		if (m_bOnFail)
		{
			iResult = m_iOnFail;
		}
		return;
	}

	// Negate if necessary
	if (bIsNeg) dVal = -dVal;

	// If out of range, limit and warn
	bool bOutOfRange = false;
	double dLimVal = dVal;
	if (dLimVal < m_dLoVal) 
	{
		dLimVal = m_dLoVal;
		bOutOfRange = true;
	}

	if (dLimVal > m_dHiVal) 
	{
		dLimVal = m_dHiVal;
		bOutOfRange = true;
	}

	// Found number, intrepolate
	double dScaleVal = (dLimVal - m_dLoVal) / (m_dHiVal - m_dLoVal) 
						* ( ((double)m_iHiDeg) - ((double)m_iLoDeg))
						+ ((double)m_iLoDeg) + 0.5;

	// have to modify this code for clock
	ASSERT (!this->m_bIsClock);

	if (dScaleVal < 0) dScaleVal += 360;
	iResult = (int) dScaleVal;

	sResult.Format( IDS_RESULTFORMAT, (long) iResult, (double) dVal );
	if (bOutOfRange)
	{
		CString str;
		str.LoadString( IDS_OUTOFRANGE );
		sResult += str;
	}

	return;
}


// Serialization
void CWebNumDial::Serialize(CArchive &ar)
{
	CDevice::Serialize( ar );

	if (ar.IsStoring())
	{
		ar << m_iResult;
		ar << m_bOnFail;
		ar << m_iOnFail;
		ar << m_iHiDeg;
		ar << m_dHiVal;
		ar << m_iLoDeg;
		ar << m_dLoVal;
		ar << m_bCalcPercent;
		ar << m_bBefore;
		ar << m_bIsClock;
		ar << m_sTriggerText;
		ar << m_bEuroFormat;
		ar << m_iNumCount;

	}
	else
	{
		ar >> m_iResult;
		ar >> m_bOnFail;
		ar >> m_iOnFail;
		ar >> m_iHiDeg;
		ar >> m_dHiVal;
		ar >> m_iLoDeg;
		ar >> m_dLoVal;
		ar >> m_bCalcPercent;
		ar >> m_bBefore;
		ar >> m_bIsClock;
		ar >> m_sTriggerText;
		ar >> m_bEuroFormat;
		ar >> m_iNumCount;
	}

}


void CWebNumDial::RemoveHTML(CString& str)
{
	bool bInHTML = false;
	CString NewStr;
	for (long i = 0; i<str.GetLength( ) ;i++)
	{
		if (str.GetAt(i)=='<')  
		{
			bInHTML = true;
			// Add space so numbers don't run together
			NewStr += " ";
		}
		if (!bInHTML) NewStr += str.GetAt(i);
		if (str.GetAt(i)=='>')  bInHTML = false;
	}

	str = NewStr;
}

void CWebNumDial::TransmitMessage()
{
	
}

